/*
 *     Module Name       : JDMSocket.h
 *
 *     Author/Location   : J.R.F. De Maeijer, Amstelveen
 *
 *     Copyright         : 2011, J.R.F. De Maeijer, Amstelveen
 *
 * ----------------------------------------------------------------------------
 * Copyright (c) 2011, J.R.F. De Maeijer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 *                            MODIFICATION HISTORY
 * ----------------------------------------------------------------------------
 * DATE        REASON                                                    AUTHOR
 * ----------------------------------------------------------------------------
 * 16-Jan-2011 Initial Release                                           J.D.M.
 * 05-Apr-2011 Renamed to JDMSocket and implemented GetStatusText()      J.D.M.
 * 15-Jul-2013 - Added functions GetMaximumMessageSizeRead() and
 *               GetMaximumMessageSizeWrite()
 *             - Renamed JDM_SOCKET to JDMSOCKET
 *             - Changed __cdecl into __stdcall at the interface         J.D.M.
 * 19-Jul-2013 - Adapted read and write functions
 *             - Added resource file V1.0.1.                             J.D.M.
 * 13-Dec-2013 - Added GetInitFileDescriptor() and moved functions
 *               GetMaximumMessageSizeRead/Write() to interface.
 * 26-Mar-2014 Added JDMSOCKET_ERROR_COULD_NOT_SET_SOCKET_OPTION         J.D.M.
 * ----------------------------------------------------------------------------
 */

#ifndef __JDMSocket_h__
#define __JDMSocket_h__

//#define STATIC_LINKING

#undef JDMSOCKET_CALL_CONV
#undef JDMSOCKET_INTERFACE_DEF_NOCC
#undef JDMSOCKET_INTERFACE_DEF

#ifdef STATIC_LINKING
#  define JDMSOCKET_CALL_CONV
#  define JDMSOCKET_INTERFACE_DEF_NOCC
#  define JDMSOCKET_INTERFACE_DEF(x)   x
#else
#  define JDMSOCKET_CALL_CONV  __stdcall
//#define JDMSOCKET_CALL_CONV  __cdecl
#  ifdef JDMSOCKET_EXPORTS
#    define JDMSOCKET_INTERFACE_DEF_NOCC __declspec(dllexport)                       /* used to export DLL names */
#    define JDMSOCKET_INTERFACE_DEF(x)   __declspec(dllexport) x JDMSOCKET_CALL_CONV /* used to export DLL names */
#  else
#    define JDMSOCKET_INTERFACE_DEF_NOCC __declspec(dllimport)                       /* used to import DLL names */
#    define JDMSOCKET_INTERFACE_DEF(x)   __declspec(dllimport) x JDMSOCKET_CALL_CONV /* used to import DLL names */
#  endif
#endif

#include <windows.h>
#include <string>
using namespace std;

#ifdef __cplusplus
extern "C" {
#endif

enum JDMSOCKET_ERRORS {
  JDMSOCKET_NO_ERROR                          =0
 ,JDMSOCKET_WAIT_AND_SEND_REMAINING_AGAIN     =-1
 ,JDMSOCKET_WAIT_AND_READ_REMAINING_AGAIN     =-2
 ,JDMSOCKET_ERROR_WSASYSNOTREADY              =-3
 ,JDMSOCKET_ERROR_WSAVERNOTSUPPORTED          =-4
 ,JDMSOCKET_ERROR_WSAEINPROGRESS              =-5
 ,JDMSOCKET_ERROR_WSAEPROCLIM                 =-6
 ,JDMSOCKET_ERROR_WSAEFAULT                   =-7
 ,JDMSOCKET_ERROR_WSANOTINITIALISED           =-8
 ,JDMSOCKET_ERROR_WSAENETDOWN                 =-9
 ,JDMSOCKET_ERROR_WSAUNKNOWN                  =-10
 ,JDMSOCKET_ERROR_GETHOSTBYNAME               =-11
 ,JDMSOCKET_ERROR_COULD_NOT_CREATE            =-12
 ,JDMSOCKET_ERROR_COULD_NOT_BIND              =-13
 ,JDMSOCKET_ERROR_COULD_NOT_CONTROL_IOMODE    =-14
 ,JDMSOCKET_ERROR_COULD_NOT_LISTEN            =-15
 ,JDMSOCKET_ERROR_COULD_NOT_CLOSE             =-16
 ,JDMSOCKET_ERROR_COULD_NOT_CONNECT           =-17
 ,JDMSOCKET_ERROR_DURING_SEND                 =-18
 ,JDMSOCKET_ERROR_COULD_NOT_SELECT            =-19
 ,JDMSOCKET_ERROR_COULD_NOT_ACCEPT            =-20
 ,JDMSOCKET_ERROR_DURING_READ                 =-21
 ,JDMSOCKET_ERROR_CONNECTION_CLOSED           =-22
 ,JDMSOCKET_ERROR_LENGTH_DATA_EXCEEDED        =-23
 ,JDMSOCKET_ERROR_NO_FILEDESCRIPTOR           =-24
 ,JDMSOCKET_ERROR_NO_DATA                     =-25
 ,JDMSOCKET_ERROR_USE_SOCKET_DEINIT           =-26
 ,JDMSOCKET_ERROR_NO_DATA_TO_WRITE            =-27
 ,JDMSOCKET_ERROR_NO_DATA_TO_READ             =-28
 ,JDMSOCKET_ERROR_COULD_NOT_SET_SOCKET_OPTION =-29
};

#ifdef __cplusplus
}

/*---------------------------------------------------------------------------*/
/* CLASS JDMSocket                                                           */
/*---------------------------------------------------------------------------*/
class JDMSocket {
  public:
    JDMSOCKET_INTERFACE_DEF (static long) InitSocketLib();
    JDMSOCKET_INTERFACE_DEF (static long) DeInitSocketLib();
    
    JDMSOCKET_INTERFACE_DEF_NOCC JDMSocket();
    JDMSOCKET_INTERFACE_DEF_NOCC ~JDMSocket();

    /* PLUG / CLIENT */
    JDMSOCKET_INTERFACE_DEF (long) Init(long Port, bool Blocking, const char* HostName); /* Plug (connects to server socket) */
    JDMSOCKET_INTERFACE_DEF (long) Read(char* RawData, long MaxDataLength, bool WhenBlockingWaitForMaxDataLength); /* Use on plug */
    JDMSOCKET_INTERFACE_DEF (long) Write(const char* RawData, long DataLength); /* Use on plug */

    /* SOCKET / SERVER */
    JDMSOCKET_INTERFACE_DEF (long) Init(long Port, bool Blocking); /* Local socket (accepts client plugs) */
    JDMSOCKET_INTERFACE_DEF (long) AcceptConnection(); /* Use on socket */
    JDMSOCKET_INTERFACE_DEF (long) CloseConnection(long& FileDescriptor); /* Use on socket */
    JDMSOCKET_INTERFACE_DEF (long) Read(long FileDescriptor, char* RawData, long MaxDataLength, bool WhenBlockingWaitForMaxDataLength); /* Use on socket */
    JDMSOCKET_INTERFACE_DEF (long) Write(long FileDescriptor, const char* RawData, long DataLength); /* Use on socket */

    JDMSOCKET_INTERFACE_DEF (long) DeInit(); /* Use on socket and plug */

    JDMSOCKET_INTERFACE_DEF (long)        GetInitFileDescriptor() {return ServerFD;}
    JDMSOCKET_INTERFACE_DEF (long)        GetMaximumMessageSizeRead(long FileDescriptor);
    JDMSOCKET_INTERFACE_DEF (long)        GetMaximumMessageSizeWrite(long FileDescriptor);
    JDMSOCKET_INTERFACE_DEF (long)        GetLastStatus();
    JDMSOCKET_INTERFACE_DEF (long)        GetLastLengthWritten();
    JDMSOCKET_INTERFACE_DEF (long)        GetLastLengthRead();
    JDMSOCKET_INTERFACE_DEF (const char*) GetStatusText(long StatusValue); /*  */
    JDMSOCKET_INTERFACE_DEF (const char*) GetErrorText(long ErrorValue); /* Local JDMSOCKET_ERRORS */

  private:
    long Listen();
    long Connect();
    long SetBlockingMode();
    long SaveStatus(long Value);
    long ReadFromSocket(long FileDescriptor, char* RawData, long MaxDataLength, bool WhenBlockingWaitForMaxDataLength);
    long WriteToSocket(long FileDescriptor, const char* RawData, long DataLength);

  private:
    long               Type;
    long               Port;
    unsigned long      BlockingMode;
    long               ServerFD;
    struct sockaddr_in ServerAddress;

    long               LastStatus;
    string             StatusText;
    string             ErrorText;
    long               LastLengthWritten;
    long               LastLengthRead;
}; /* CLASS JDMSocket */

#endif

#endif
